@@ -44,9 +44,14 @@ GFile *       ostree_repo_get_path (OstreeRepo  *self);
 
 typedef enum {
   OSTREE_REPO_MODE_BARE,
-  OSTREE_REPO_MODE_ARCHIVE
+  OSTREE_REPO_MODE_ARCHIVE,
+  OSTREE_REPO_MODE_ARCHIVE_Z
 } OstreeRepoMode;
 
+gboolean       ostree_repo_mode_from_string (const char      *mode,
+                                             OstreeRepoMode  *out_mode,
+                                             GError         **error);
+
 OstreeRepoMode ostree_repo_get_mode (OstreeRepo  *self);
 
 GFile *       ostree_repo_get_tmpdir (OstreeRepo  *self);
@@ -90,35 +95,35 @@ gboolean      ostree_repo_has_object (OstreeRepo           *self,
                                       GCancellable         *cancellable,
                                       GError              **error);
 
-gboolean      ostree_repo_stage_object (OstreeRepo       *self,
-                                        OstreeObjectType  objtype,
-                                        const char       *expected_checksum,
-                                        GInputStream     *content,
-                                        GCancellable     *cancellable,
-                                        GError          **error);
+gboolean      ostree_repo_stage_metadata (OstreeRepo        *self,
+                                          OstreeObjectType   objtype,
+                                          const char        *expected_checksum,
+                                          GVariant          *object,
+                                          guchar           **out_csum,
+                                          GCancellable      *cancellable,
+                                          GError           **error);
 
-gboolean      ostree_repo_stage_file_object (OstreeRepo       *self,
-                                             const char       *expected_checksum,
-                                             GInputStream     *content,
-                                             guint64           content_length,
-                                             GCancellable     *cancellable,
-                                             GError          **error);
+gboolean      ostree_repo_stage_content (OstreeRepo       *self,
+                                         const char       *expected_checksum,
+                                         GInputStream     *content,
+                                         guint64           content_length,
+                                         guchar          **out_csum,
+                                         GCancellable     *cancellable,
+                                         GError          **error);
 
-gboolean      ostree_repo_stage_object_trusted (OstreeRepo   *self,
-                                                OstreeObjectType objtype,
-                                                const char   *checksum,
-                                                gboolean          store_if_packed,
-                                                GInputStream     *content,
-                                                GCancellable *cancellable,
-                                                GError      **error);
+gboolean      ostree_repo_stage_metadata_trusted (OstreeRepo        *self,
+                                                  OstreeObjectType   objtype,
+                                                  const char        *checksum,
+                                                  GVariant          *object,
+                                                  GCancellable      *cancellable,
+                                                  GError           **error);
 
-gboolean      ostree_repo_stage_file_object_trusted (OstreeRepo   *self,
-                                                     const char   *checksum,
-                                                     gboolean          store_if_packed,
-                                                     GInputStream     *content,
-                                                     guint64           content_length,
-                                                     GCancellable *cancellable,
-                                                     GError      **error);
+gboolean      ostree_repo_stage_content_trusted (OstreeRepo       *self,
+                                                 const char       *checksum,
+                                                 GInputStream     *content,
+                                                 guint64           content_length,
+                                                 GCancellable     *cancellable,
+                                                 GError          **error);
 
 gboolean      ostree_repo_resolve_rev (OstreeRepo  *self,
                                        const char  *rev,
@@ -149,26 +154,11 @@ gboolean      ostree_repo_load_variant (OstreeRepo  *self,
                                         GVariant     **out_variant,
                                         GError       **error);
 
-gboolean      ostree_repo_load_pack_index (OstreeRepo    *self,
-                                           const char    *pack_checksum, 
-                                           gboolean       is_meta,
-                                           GVariant     **out_variant,
-                                           GCancellable  *cancellable,
-                                           GError       **error);
-
-gboolean      ostree_repo_load_pack_data  (OstreeRepo    *self,
-                                           const char    *pack_checksum,
-                                           guchar       **out_data,
-                                           GCancellable  *cancellable,
-                                           GError       **error);
-
-gboolean ostree_repo_map_pack_file (OstreeRepo    *self,
-                                    const char    *sha256,
-                                    gboolean       is_meta,
-                                    guchar       **out_data,
-                                    guint64       *out_len,
-                                    GCancellable  *cancellable,
-                                    GError       **error);
+gboolean      ostree_repo_load_variant_if_exists (OstreeRepo  *self,
+                                                  OstreeObjectType expected_type,
+                                                  const char    *sha256, 
+                                                  GVariant     **out_variant,
+                                                  GError       **error);
 
 gboolean ostree_repo_load_file (OstreeRepo         *self,
                                 const char         *entry_sha256,
@@ -238,65 +228,6 @@ gboolean      ostree_repo_stage_commit (OstreeRepo   *self,
                                         GCancellable *cancellable,
                                         GError      **error);
 
-gboolean ostree_repo_regenerate_pack_index (OstreeRepo       *self,
-                                            GCancellable     *cancellable,
-                                            GError          **error);
-
-gboolean     ostree_repo_add_pack_file (OstreeRepo       *self,
-                                        const char       *checksum,
-                                        gboolean          is_meta,
-                                        GFile            *pack_index_path,
-                                        GFile            *pack_data_path,
-                                        GCancellable     *cancellable,
-                                        GError          **error);
-
-gboolean     ostree_repo_resync_cached_remote_pack_indexes (OstreeRepo       *self,
-                                                            const char       *remote_name,
-                                                            GFile            *superindex_path,
-                                                            GPtrArray       **out_cached_meta_indexes,
-                                                            GPtrArray       **out_cached_data_indexes,
-                                                            GPtrArray       **out_uncached_meta_indexes,
-                                                            GPtrArray       **out_uncached_data_indexes,
-                                                            GCancellable     *cancellable,
-                                                            GError          **error);
-
-gboolean     ostree_repo_clean_cached_remote_pack_data (OstreeRepo       *self,
-                                                        const char       *remote_name,
-                                                        GCancellable     *cancellable,
-                                                        GError          **error);
-
-gboolean     ostree_repo_map_cached_remote_pack_index (OstreeRepo       *self,
-                                                       const char       *remote_name,
-                                                       const char       *pack_checksum,
-                                                       gboolean          is_meta,
-                                                       GVariant        **out_variant,
-                                                       GCancellable     *cancellable,
-                                                       GError          **error);
-
-gboolean     ostree_repo_add_cached_remote_pack_index (OstreeRepo       *self,
-                                                       const char       *remote_name,
-                                                       const char       *pack_checksum,
-                                                       gboolean          is_meta,
-                                                       GFile            *cached_path,
-                                                       GCancellable     *cancellable,
-                                                       GError          **error);
-
-gboolean     ostree_repo_get_cached_remote_pack_data (OstreeRepo       *self,
-                                                      const char       *remote_name,
-                                                      const char       *pack_checksum,
-                                                      gboolean          is_meta,
-                                                      GFile           **out_cached_path,
-                                                      GCancellable     *cancellable,
-                                                      GError          **error);
-
-gboolean     ostree_repo_take_cached_remote_pack_data (OstreeRepo       *self,
-                                                       const char       *remote_name,
-                                                       const char       *pack_checksum,
-                                                       gboolean          is_meta,
-                                                       GFile            *cached_path,
-                                                       GCancellable     *cancellable,
-                                                       GError          **error);
-
 typedef enum {
   OSTREE_REPO_CHECKOUT_MODE_NONE = 0,
   OSTREE_REPO_CHECKOUT_MODE_USER = 1
@@ -307,15 +238,21 @@ typedef enum {
   OSTREE_REPO_CHECKOUT_OVERWRITE_UNION_FILES = 1
 } OstreeRepoCheckoutOverwriteMode;
 
+void
+ostree_repo_checkout_tree_async (OstreeRepo               *self,
+                                 OstreeRepoCheckoutMode    mode,
+                                 OstreeRepoCheckoutOverwriteMode    overwrite_mode,
+                                 GFile                    *destination,
+                                 OstreeRepoFile           *source,
+                                 GFileInfo                *source_info,
+                                 GCancellable             *cancellable,
+                                 GAsyncReadyCallback       callback,
+                                 gpointer                  user_data);
+
 gboolean
-ostree_repo_checkout_tree (OstreeRepo               *self,
-                           OstreeRepoCheckoutMode    mode,
-                           OstreeRepoCheckoutOverwriteMode    overwrite_mode,
-                           GFile                    *destination,
-                           OstreeRepoFile           *source,
-                           GFileInfo                *source_info,
-                           GCancellable             *cancellable,
-                           GError                  **error);
+ostree_repo_checkout_tree_finish (OstreeRepo               *self,
+                                  GAsyncResult             *result,
+                                  GError                  **error);
 
 gboolean       ostree_repo_read_commit (OstreeRepo *self,
                                         const char *rev,
@@ -343,12 +280,6 @@ gboolean ostree_repo_list_objects (OstreeRepo                  *self,
                                    GCancellable                *cancellable,
                                    GError                     **error);
 
-gboolean ostree_repo_list_pack_indexes (OstreeRepo              *self,
-                                        GPtrArray              **out_meta_indexes,
-                                        GPtrArray              **out_data_indexes,
-                                        GCancellable            *cancellable,
-                                        GError                 **error);
-
 G_END_DECLS
 
 #endif /* _OSTREE_REPO */
